@@ -5,7 +5,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2020	PgPool Global Development Group
+ * Copyright (c) 2003-2017	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -26,6 +26,7 @@
 #include "context/pool_session_context.h"
 #include "utils/pool_stream.h"
 #include "pool_config.h"
+#include "auth/pool_hba.h"
 #include "auth/pool_passwd.h"
 #include "utils/elog.h"
 #include "utils/palloc.h"
@@ -218,7 +219,7 @@ int pool_do_auth(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *cp)
 		/* If MD5 auth is not active in pool_hba.conf, it cannot be
 		 * used with other than raw mode.
 		 */
-		if (frontend->auth_method != uaMD5 && !RAW_MODE && NUM_BACKENDS > 1)
+		if ((frontend->pool_hba == NULL || frontend->pool_hba->auth_method != uaMD5) && !RAW_MODE && NUM_BACKENDS > 1)
 		{
 			pool_send_error_message(frontend, protoMajor, AUTHFAIL_ERRORCODE,
 									"MD5 authentication is unsupported in replication and master-slave modes.",
@@ -635,7 +636,7 @@ static int do_clear_text_password(POOL_CONNECTION *backend, POOL_CONNECTION *fro
 	pool_read(backend, &kind, sizeof(kind));
 
 	/* if authenticated, save info */
-	if (kind == 0)
+	if (!reauth && kind == 0)
 	{
 		if (IS_MASTER_NODE_ID(backend->db_node_id))
 		{
@@ -785,7 +786,7 @@ static int do_crypt(POOL_CONNECTION *backend, POOL_CONNECTION *frontend, int rea
 	pool_read(backend, &kind, sizeof(kind));
 
 	/* if authenticated, save info */
-	if (kind == 0)
+	if (!reauth && kind == 0)
 	{
 		int msglen;
 
@@ -820,7 +821,7 @@ static int do_md5(POOL_CONNECTION *backend, POOL_CONNECTION *frontend, int reaut
 	char encbuf[POOL_PASSWD_LEN+1];
 	char *pool_passwd = NULL;
 
-	if (NUM_BACKENDS > 1)
+	if (!RAW_MODE && NUM_BACKENDS > 1)
 	{
 		/* Read password entry from pool_passwd */
 		pool_passwd = pool_get_passwd(frontend->username);
@@ -1130,12 +1131,10 @@ int pool_read_message_length(POOL_CONNECTION_POOL *cp)
 				 errdetail("slot: %d length: %d", i, length)));
 
 		if (length != length0)
-		{
-			ereport(LOG,
-					(errmsg("unable to read message length"),
-					 errdetail("message length (%d) in slot %d does not match with slot 0(%d)", length, i, length0)));
-			return -1;
-		}
+            ereport(ERROR,
+                    (errmsg("unable to read message length"),
+                     errdetail("message length (%d) in slot %d does not match with slot 0(%d)", length, i, length0)));
+            
 	}
 
 	if (length0 < 0)
