@@ -4,7 +4,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2019	PgPool Global Development Group
+ * Copyright (c) 2003-2018	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -86,7 +86,6 @@ static void pcp_worker_will_go_down(int code, Datum arg);
 static void  do_pcp_flush(PCP_CONNECTION *frontend);
 static void  do_pcp_read(PCP_CONNECTION *pc, void *buf, int len);
 
-
 /* 
  * main entry pont of pcp worker child process
  */
@@ -102,6 +101,7 @@ pcp_worker_main(int port)
 	struct timeval uptime;
 	char tos;
 	int rsize;
+	char *buf = NULL;
 
 	ereport(DEBUG1,
 			(errmsg("I am PCP worker child with pid:%d",getpid())));
@@ -158,8 +158,6 @@ pcp_worker_main(int port)
 	
 	for(;;)
 	{
-		char *buf = NULL;
-
 		MemoryContextSwitchTo(PCPMemoryContext);
 		MemoryContextResetAndDeleteChildren(PCPMemoryContext);
 
@@ -502,14 +500,14 @@ static int pool_detach_node(int node_id, bool gracefully)
 {
 	if (!gracefully)
 	{
-		degenerate_backend_set_ex(&node_id, 1, true, false, true, 0);
+		degenerate_backend_set_ex(&node_id, 1, REQ_DETAIL_SWITCHOVER | REQ_DETAIL_CONFIRMED, true, false);
 		return 0;
 	}
 
 	/* Check if the NODE DOWN can be executed on
 	 * the given node id.
 	 */
-	degenerate_backend_set_ex(&node_id, 1, true, true, true, 0);
+	degenerate_backend_set_ex(&node_id, 1, REQ_DETAIL_SWITCHOVER | REQ_DETAIL_CONFIRMED, true, true);
 
 	/*
 	 * Wait until all frontends exit
@@ -529,7 +527,7 @@ static int pool_detach_node(int node_id, bool gracefully)
 	/*
 	 * Now all frontends have gone. Let's do failover.
 	 */
-	degenerate_backend_set_ex(&node_id, 1, true, false, true, 0);
+	degenerate_backend_set_ex(&node_id, 1, REQ_DETAIL_SWITCHOVER | REQ_DETAIL_CONFIRMED, false, false);
 
 	/*
 	 * Wait for failover completed.
@@ -556,7 +554,7 @@ static int pool_promote_node(int node_id, bool gracefully)
 {
 	if (!gracefully)
 	{
-		promote_backend(node_id, false);	/* send promote request */
+		promote_backend(node_id, REQ_DETAIL_CONFIRMED);	/* send promote request */
 		return 0;
 	}
 
@@ -576,7 +574,7 @@ static int pool_promote_node(int node_id, bool gracefully)
 	/*
 	 * Now all frontends have gone. Let's do failover.
 	 */
-	promote_backend(node_id, false);		/* send promote request */
+	promote_backend(node_id, REQ_DETAIL_CONFIRMED);		/* send promote request */
 
 	/*
 	 * Wait for failover completed.
@@ -811,8 +809,10 @@ inform_node_info(PCP_CONNECTION *frontend,char *buf)
 	char port_str[6];
 	char status[2];
 	char weight_str[20];
+	char role_str[10];
 	char code[] = "CommandComplete";
 	BackendInfo *bi = NULL;
+	SERVER_ROLE role;
 
 	node_id = atoi(buf);
 
@@ -830,6 +830,22 @@ inform_node_info(PCP_CONNECTION *frontend,char *buf)
 	snprintf(port_str, sizeof(port_str), "%d", bi->backend_port);
 	snprintf(status, sizeof(status), "%d", bi->backend_status);
 	snprintf(weight_str, sizeof(weight_str), "%f", bi->backend_weight);
+
+	if (STREAM)
+	{
+		if (Req_info->primary_node_id == node_id)
+			role = ROLE_PRIMARY;
+		else
+			role = ROLE_STANDBY;
+	}
+	else
+	{
+		if (Req_info->master_node_id == node_id)
+			role = ROLE_MASTER;
+		else
+			role = ROLE_SLAVE;
+	}
+	snprintf(role_str, sizeof(role_str), "%d", role);
 	
 	pcp_write(frontend, "i", 1);
 	wsize = htonl(sizeof(code) +
@@ -837,6 +853,7 @@ inform_node_info(PCP_CONNECTION *frontend,char *buf)
 				  strlen(port_str)+1 +
 				  strlen(status)+1 +
 				  strlen(weight_str)+1 +
+				  strlen(role_str)+1 +
 				  sizeof(int));
 	pcp_write(frontend, &wsize, sizeof(int));
 	pcp_write(frontend, code, sizeof(code));
@@ -844,6 +861,8 @@ inform_node_info(PCP_CONNECTION *frontend,char *buf)
 	pcp_write(frontend, port_str, strlen(port_str)+1);
 	pcp_write(frontend, status, strlen(status)+1);
 	pcp_write(frontend, weight_str, strlen(weight_str)+1);
+
+	pcp_write(frontend, role_str, strlen(role_str)+1);
 	do_pcp_flush(frontend);
 }
 
@@ -910,7 +929,7 @@ process_attach_node(PCP_CONNECTION *frontend,char *buf)
 			(errmsg("PCP: processing attach node"),
 			 errdetail("attaching Node ID %d", node_id)));
 
-	send_failback_request(node_id,true, false);
+	send_failback_request(node_id,true, REQ_DETAIL_CONFIRMED);
 
 	pcp_write(frontend, "c", 1);
 	wsize = htonl(sizeof(code) + sizeof(int));
@@ -1289,4 +1308,3 @@ static void pcp_worker_will_go_down(int code, Datum arg)
 	POOL_SETMASK(&UnBlockSig);
 	
 }
-
